/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base;

import org.apache.ws.notification.WsnRuntime;
import org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertyCallback;
import org.apache.ws.resource.properties.impl.CallbackFailedException;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionDialectsDocument;
import javax.xml.namespace.QName;

/**
 * A callback for the "TopicExpressionDialects" resource property from the WS-BaseNotification
 * NotificationProducer portType.
 */
public class TopicExpressionDialectsResourcePropertyCallback
   implements ResourcePropertyCallback
{
   /**
    * Initializes the value of the wsnt:TopicExpressionDialects resource property so that it
    * reflects the topic expression dialects supported by the topic expression engine.
    *
    * @param prop the wsnt:TopicExpressionDialects resource property
    *
    * @return the wsnt:TopicExpressionDialects resource property
    *
    * @throws org.apache.ws.resource.properties.impl.CallbackFailedException
    */
   public ResourceProperty refreshProperty( ResourceProperty prop )
   throws CallbackFailedException
   {
      QName propName = prop.getMetaData(  ).getName(  );
      if ( !propName.equals( NotificationProducerPortType.PROP_QNAME_TOPIC_EXPRESSION_DIALECTS ) )
      {
         throw new IllegalArgumentException( "Unsupported property: " + propName );
      }

      if ( prop.isEmpty(  ) )
      {
         String[] supportedDialects =
            WsnRuntime.getRuntime(  ).getTopicExpressionEngine(  ).getSupportedDialects(  );
         for ( int i = 0; i < supportedDialects.length; i++ )
         {
            TopicExpressionDialectsDocument topicExpressionDialectsDocument =
               TopicExpressionDialectsDocument.Factory.newInstance(  );
            topicExpressionDialectsDocument.setTopicExpressionDialects( supportedDialects[i].toString(  ) );
            prop.add( topicExpressionDialectsDocument );
         }
      }

      return prop;
   }
}