/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base;

import org.apache.ws.UnsupportedNamespaceException;
import org.apache.ws.notification.base.v2004_06.porttype.NotificationProducerPortType;
import org.apache.ws.notification.topics.TopicSet;
import org.apache.ws.notification.topics.TopicsTypeWriter;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import org.apache.ws.resource.properties.ResourceProperty;
import org.apache.ws.resource.properties.ResourcePropertyCallback;
import org.apache.ws.resource.properties.impl.CallbackFailedException;
import javax.xml.namespace.QName;

/**
 * A callback for the Topic resource property from the WS-BaseNotification NotificationProducer portType.
 */
public class TopicResourcePropertyCallback
   implements ResourcePropertyCallback
{
   private TopicSet m_topicSet;

   /**
    * Creates a new {@link TopicResourcePropertyCallback} for the specified topic set.
    *
    * @param topicSet the producer's topic set
    */
   public TopicResourcePropertyCallback( TopicSet topicSet )
   {
      m_topicSet = topicSet;
   }

   /**
    * Refreshes the value of the wsnt:Topic resource property so that it reflects the topics contained in the producer
    * resource's topic set.
    *
    * @param prop the wsnt:Topic resource property
    *
    * @return the wsnt:Topic resource property
    *
    * @throws org.apache.ws.resource.properties.impl.CallbackFailedException
    *
    */
   public ResourceProperty refreshProperty( ResourceProperty prop )
   throws CallbackFailedException
   {
      QName propName = prop.getMetaData(  ).getName(  );
      if ( !propName.equals( NotificationProducerPortType.PROP_QNAME_TOPIC ) )
      {
         throw new IllegalArgumentException( "Unsupported property: " + propName );
      }

      if ( prop.isEmpty(  ) || !m_topicSet.isFixed(  ) )
      {
         prop.clear(  );
         TopicExpression[] topicExprs = m_topicSet.getTopicExpressions(  );
         for ( int i = 0; i < topicExprs.length; i++ )
         {
            try
            {
               prop.add( TopicsTypeWriter.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toXmlObject( topicExprs[i],
                                                                                                         NotificationProducerPortType.PROP_QNAME_TOPIC ) );
            }
            catch ( UnsupportedNamespaceException une )
            {
               throw new CallbackFailedException( une );
            }
         }
      }

      return prop;
   }
}