/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base;

import org.apache.ws.resource.NamespaceVersionHolder;

/**
 * An extension of the {@link NamespaceVersionHolder}  which adds WSN-specific
 * getters for the WSN namespaces.
 *
 * Implementations of this interface are used to provide an object which contains all the
 * namespaces supported by a given Resource.  This allows us to group the properly associated versioned
 * namespaces together in one place.
 *
 * @author Sal Campana
 */
public interface WsnNamespaceVersionHolder
   extends NamespaceVersionHolder
{
   /**
    * Returns the BaseNotification namespace
    * @return  BaseNotification namespace
    */
   String getBaseNotificationXsdNamespace(  );

   /**
    * Returns the Brokered Notification namespace
    * @return Brokered Notification namespace
    */
   String getBrokeredNotificationXsdNamespace(  );

   /**
    * Returns the namespace for the Topics Spec
    *
    * @return  Topic namespace
    */
   String getTopicsXsdNamespace(  );
}