/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.impl;

import org.apache.ws.addressing.EndpointReference;
import org.apache.ws.notification.base.CustomBaseNotificationConstants;
import org.apache.ws.notification.base.Subscription;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.resource.NamespaceVersionHolder;
import org.apache.ws.resource.ResourceException;
import org.apache.ws.resource.impl.AbstractResourceHome;
import javax.xml.namespace.QName;
import java.util.Map;

/**
 * The builtin home for 2004/06 {@link Subscription} resources.
 *
 * @author Ian Springer
 */
public class SubscriptionHome
   extends AbstractResourceHome
{
   /**
    * The WSDL Port name associated with the resource. This is useful for building EPRs.
    */
   public static final String                  PORT_NAME          = "SubscriptionManager";
   private static final NamespaceVersionHolder SPEC_NAMESPACE_SET = new WsnNamespaceVersionHolderImpl(  );

   /**
    * Map containing all FilesystemResource instances - this map <em>must</em> be static for compatibility with certain
    * JNDI providers.
    */
   private static Map s_resources;

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public NamespaceVersionHolder getNamespaceSet(  )
   {
      return SPEC_NAMESPACE_SET;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getPortType(  )
   {
      return CustomBaseNotificationConstants.PORT_TYPE;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getServiceName(  )
   {
      return CustomBaseNotificationConstants.SERVICE_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getServicePortName(  )
   {
      return PORT_NAME;
   }

   /**
    * DOCUMENT_ME
    *
    * @param producerRef      DOCUMENT_ME
    * @param consumerRef      DOCUMENT_ME
    * @param topicExpr        DOCUMENT_ME
    * @return DOCUMENT_ME
    *
    * @throws ResourceException DOCUMENT_ME
    */
   public Subscription create( EndpointReference producerRef,
                               EndpointReference consumerRef,
                               TopicExpression   topicExpr )
   throws ResourceException
   {
      Subscription subscription = new SubscriptionResource( producerRef, consumerRef, topicExpr );
      add( subscription );
      return subscription;
   }

   /**
    * Returns a map of all Subscription instances. Used by the {@link AbstractResourceHome} superclass.
    */
   protected final synchronized Map getResourceMap(  )
   {
      if ( s_resources == null )
      {
         s_resources = AbstractResourceHome.createResourceMap( m_resourceIsPersistent );
      }

      return s_resources;
   }
}