/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.base.v2004_06.porttype;

import org.apache.ws.notification.base.v2004_06.BaseNotificationConstants;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.GetCurrentMessageDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.GetCurrentMessageResponseDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeDocument;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.SubscribeResponseDocument;
import javax.xml.namespace.QName;

/**
 * The WS-BaseNotification 2004/06 NotificationProducer portType.
 */
public interface NotificationProducerPortType
{
   /**
    * QName of NotificationProducer.
    */
   QName NAME =
      new QName( BaseNotificationConstants.NSURI_WSNT_WSDL, "NotificationProducer",
                 BaseNotificationConstants.NSPREFIX_WSNT_WSDL );

   /**
    * QName of  NotificationProducerRP Resource Properties Document.
    */
   QName PROPS_DOC_NAME =
      new QName( BaseNotificationConstants.NSURI_WSNT_SCHEMA, "NotificationProducerRP",
                 BaseNotificationConstants.NSPREFIX_WSNT_SCHEMA );

   /**
    * Topic property name.
    */
   QName PROP_QNAME_TOPIC =
      new QName( BaseNotificationConstants.NSURI_WSNT_SCHEMA, "Topic",
                 BaseNotificationConstants.NSPREFIX_WSNT_SCHEMA );

   /**
    * FixedTopicSet property name.
    */
   QName PROP_QNAME_FIXED_TOPIC_SET =
      new QName( BaseNotificationConstants.NSURI_WSNT_SCHEMA, "FixedTopicSet",
                 BaseNotificationConstants.NSPREFIX_WSNT_SCHEMA );

   /**
    * TopicExpressionDialects property name.
    */
   QName PROP_QNAME_TOPIC_EXPRESSION_DIALECTS =
      new QName( BaseNotificationConstants.NSURI_WSNT_SCHEMA, "TopicExpressionDialects",
                 BaseNotificationConstants.NSPREFIX_WSNT_SCHEMA );

   /**
    * Method used for retrieving the current message from the NotificationProducer.
    *
    * @param requestDoc
    * @return  GetCurrentMessageResponseDocument
    */
   GetCurrentMessageResponseDocument getCurrentMessage( GetCurrentMessageDocument requestDoc );

   /**
    * Method used to subscribe for notifications.
    *
    * @param requestDoc
    * @return SubscribeResponseDocument
    */
   SubscribeResponseDocument subscribe( SubscribeDocument requestDoc );
}