/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics;

import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import org.apache.xmlbeans.XmlObject;
import org.springframework.core.io.DefaultResourceLoader;
import org.springframework.core.io.Resource;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

/**
 * A registry for topic namespace definitions.
 *
 * @author Ian Springer
 */
public class TopicNamespaceRegistry
{
   /*
    * maps targetNamespace <String> to <TopicNamespace>
    */
   private final Map m_topicNsMap = Collections.synchronizedMap( new HashMap(  ) );

   /**
    * Creates a new {@link TopicNamespaceRegistry} object.
    */
   public TopicNamespaceRegistry(  )
   {
   }

   /**
    * Creates a new {@link TopicNamespaceRegistry} object.
    *
    * @param topicNsSet DOCUMENT_ME
    */
   public TopicNamespaceRegistry( Set topicNsSet )
   {
      initTopicNamespaceMap( topicNsSet );
   }

   /**
    * DOCUMENT_ME
    *
    * @param targetNamespace DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public TopicNamespace getTopicNamespace( String targetNamespace )
   {
      return (TopicNamespace) m_topicNsMap.get( targetNamespace.toString(  ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicNs DOCUMENT_ME
    */
   public void addTopicNamespace( TopicNamespace topicNs )
   {
      m_topicNsMap.put( ( topicNs.getTargetNamespace(  ) != null ) ? topicNs.getTargetNamespace(  ).toString(  ) : "",
                        topicNs );
   }

   /**
    * Loads each of the TopicNamespace documents from the URLs in the specified set and
    * adds them to this registry.
    *
    * @param topicNsSet a set of Strings representing URLs of TopicNamespace documents
    */
   public void initTopicNamespaceMap( Set topicNsSet )
   {
      for ( Iterator iterator = topicNsSet.iterator(  ); iterator.hasNext(  ); )
      {
         String   location = (String) iterator.next(  );
         Resource resource;
         try
         {
            resource = new DefaultResourceLoader(  ).getResource( location );
         }
         catch ( RuntimeException re )
         {
            System.err.println( "ERROR: " + location
                                + " is not a valid URL - an associated topic namespace will not be added to the registry." );
            continue;
         }

         TopicNamespace topicNs = null;
         try
         {
            topicNs =
               TopicsTypeReader.newInstance( TopicsConstants.NSURI_WSTOP_SCHEMA ).toTopicNamespace( XmlObject.Factory
                                                                                                    .parse( resource
                                                                                                            .getInputStream(  ) ) );
         }
         catch ( Exception e )
         {
            System.err.println( "ERROR: Unable to load topic namespace document from URL " + location
                                + " - an associated topic namespace will not be added to the registry." );
            e.printStackTrace(  );
            continue;
         }

         m_topicNsMap.put( topicNs.getTargetNamespace(  ),
                           topicNs );
      }
   }
}