/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.expression;

import org.apache.ws.notification.topics.expression.impl.AbstractTopicExpression;
import org.apache.ws.notification.topics.v2004_06.TopicsConstants;
import javax.xml.namespace.QName;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

/**
 * A Concrete TopicExpression as defined by the WS-Topics specification.
 */
public class ConcreteTopicExpression
   extends AbstractTopicExpression
{
   private transient QName m_topicPath;

   /**
    * Creates a new {@link ConcreteTopicExpression} object.
    *
    * @param topicPath DOCUMENT_ME
    */
   public ConcreteTopicExpression( QName topicPath )
   {
      super( TopicsConstants.TOPIC_EXPR_DIALECT_CONCRETE, topicPath );
      m_topicPath = topicPath;
   }

   /**
    * Creates a new {@link ConcreteTopicExpression} object.
    */
   protected ConcreteTopicExpression(  )
   {
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public QName getTopicPath(  )
   {
      return m_topicPath;
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean equals( Object obj )
   {
      return ( obj != null ) && obj instanceof ConcreteTopicExpression && ( obj.hashCode(  ) == hashCode(  ) );
   }

   private void readObject( ObjectInputStream in )
   throws IOException, 
          ClassNotFoundException
   {
      String qname = (String) in.readObject(  );
      m_topicPath = QName.valueOf( qname );
      setContent( m_topicPath );
      setDialect( TopicsConstants.TOPIC_EXPR_DIALECT_CONCRETE );
   }

   private void writeObject( ObjectOutputStream out )
   throws IOException
   {
      out.writeObject( m_topicPath.toString(  ) );
   }
}