/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.expression.impl;

import org.apache.commons.lang.StringUtils;
import org.apache.ws.notification.topics.TopicNamespace;
import org.apache.ws.notification.topics.TopicNamespaceRegistry;
import org.apache.ws.notification.topics.expression.InvalidTopicExpressionException;
import org.apache.ws.notification.topics.expression.TopicExpressionEvaluator;
import org.apache.ws.notification.topics.expression.TopicExpressionResolutionException;
import javax.xml.namespace.QName;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

/**
 * An abstract class that provides functionality that is shared by the {@link TopicExpressionEvaluator}s for all three
 * WS-Topics topic expression dialects.
 *
 * @author Ian Springer
 */
public abstract class AbstractTopicExpressionEvaluator
   implements TopicExpressionEvaluator
{
   /** DOCUMENT_ME */
   protected static final Map RESULTS_CACHE = Collections.synchronizedMap( new HashMap(  ) );

   /**
    * DOCUMENT_ME
    *
    * @param topicPath DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws TopicExpressionResolutionException
    *          DOCUMENT_ME
    */
   protected TopicNamespace getTopicNamespace( TopicNamespaceRegistry topicNsRegistry,
                                               QName                  topicPath )
   throws TopicExpressionResolutionException
   {
      TopicNamespace topicNs = topicNsRegistry.getTopicNamespace( topicPath.getNamespaceURI(  ) );
      if ( topicNs == null )
      {
         throw new TopicExpressionResolutionException( "Topic path '" + topicPath
                                                       + "' references an unknown TopicNamespace with target namespace '"
                                                       + topicPath.getNamespaceURI(  ) + "'." );
      }

      return topicNs;
   }

   /**
    * Performs basic validation of the specified topic path.
    *
    * @param topicPath a topicPath
    *
    * @throws InvalidTopicExpressionException
    */
   protected void validateTopicPath( QName topicPath )
   throws InvalidTopicExpressionException
   {
      String localPart = topicPath.getLocalPart(  );
      if ( StringUtils.isEmpty( localPart ) || !StringUtils.containsNone( localPart, " \t\n\r\f" ) )
      {
         throw new InvalidTopicExpressionException( "Topic path '" + topicPath + "' contains whitespace." );
      }

      if ( localPart.indexOf( ':' ) != -1 )
      {
         throw new InvalidTopicExpressionException( "Topic path '" + topicPath + "' contains a colon." );
      }
   }
}