/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.impl;

import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicNamespace;
import org.apache.ws.notification.topics.expression.TopicExpression;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * An implementation of a <code>Topic</code>.
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public class TopicImpl
   implements Topic
{
   /**
    * DOCUMENT_ME
    */
   protected String m_name;

   /**
    * DOCUMENT_ME
    */
   protected Map m_subTopics = new HashMap(  );

   /**
    * DOCUMENT_ME
    */
   protected TopicExpression m_reference;

   /*
    * List of the components of the topic path
    */

   /** DOCUMENT_ME */
   protected List m_topicPath = new LinkedList(  ); //

   /**
    * DOCUMENT_ME
    */
   protected TopicNamespace m_topicNs;

   /**
    * Create a topic with the given name
    *
    * @param name The name of the created topic
    */
   public TopicImpl( String name )
   {
      m_name = name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getName(  )
   {
      return m_name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean isReference(  )
   {
      return m_reference != null;
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicName DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Topic getTopic( String topicName )
   {
      return (Topic) m_subTopics.get( topicName );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicNs DOCUMENT_ME
    */
   public void setTopicNamespace( TopicNamespace topicNs )
   {
      m_topicNs = topicNs;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public TopicNamespace getTopicNamespace(  )
   {
      return m_topicNs;
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicPath DOCUMENT_ME
    */
   public void setTopicPath( List topicPath )
   {
      m_topicPath = topicPath;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public List getTopicPath(  )
   {
      return m_topicPath;
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicPath DOCUMENT_ME
    */
   public void setTopicReference( TopicExpression topicPath )
   {
      this.m_reference = topicPath;
   }

   /**
    * Get the topic expression for the topic(s) that this topic references.
    *
    * @return The topic expression that this topic reference or null if this topic is not a reference.
    */
   public TopicExpression getTopicReference(  )
   {
      return m_reference;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Topic addTopic( String name )
   {
      return addTopic( new TopicImpl( name ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean containsTopic( String name )
   {
      return m_subTopics.containsKey( name );
   }

   /**
    * @param obj
    *
    * @return
    */
   public boolean equals( Object obj )
   {
      return ( obj != null ) && obj instanceof TopicImpl && ( obj.hashCode(  ) == hashCode(  ) );
   }

   /**
    * @return
    */
   public int hashCode(  )
   {
      return toString(  ).hashCode(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void removeTopic( String name )
   {
      m_subTopics.remove( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topic DOCUMENT_ME
    */
   public void removeTopic( Topic topic )
   {
      m_subTopics.remove( topic.getName(  ) );
   }

   /**
    * Returns a String representation of this Topic in following format: "{" + target_namespace + "}" + topic_path. If
    * the target namespace is "" (i.e. the ad-hoc namespace), only the topic path is returned. For example:
    * "{http://example.org/topicSpace/example1}t1/t3" or "t7/t8".
    *
    * @return a String representation of this Topic
    */
   public String toString(  )
   {
      StringBuffer strBuf   = new StringBuffer(  );
      String       targetNs = getTopicNamespace(  ).getTargetNamespace(  );
      if ( ( targetNs != null ) && !targetNs.equals( "" ) )
      {
         strBuf.append( '{' );
         strBuf.append( targetNs );
         strBuf.append( '}' );
      }

      if ( ( m_topicPath != null ) && !m_topicPath.isEmpty(  ) )
      {
         for ( int i = 0; i < m_topicPath.size(  ); i++ )
         {
            strBuf.append( m_topicPath.get( i ) );
            strBuf.append( '/' );
         }

         strBuf.setLength( strBuf.length(  ) - 1 ); // chop off trailing slash
      }

      return strBuf.toString(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Iterator topicIterator(  )
   {
      return m_subTopics.values(  ).iterator(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topic DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   private Topic addTopic( Topic topic )
   {
      if ( m_reference != null )
      {
         throw new IllegalStateException( "Cannot modify the TopicExpression reference by adding an additional Topic." );
      }

      m_subTopics.put( topic.getName(  ),
                       topic );
      topic.setTopicNamespace( m_topicNs );
      List topicPath = new LinkedList(  );
      topicPath.addAll( m_topicPath );
      topicPath.add( topic.getName(  ) );
      topic.setTopicPath( topicPath );
      return topic;
   }
}