/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.impl;

import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicNamespace;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * An implementation of a <code>TopicNamespace</code>.
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public class TopicNamespaceImpl
   implements TopicNamespace
{
   private String  m_targetNs;
   private String  m_name;
   private boolean m_final;
   private Map     m_rootTopicMap = Collections.synchronizedMap( new HashMap(  ) );

   /**
    * Creates a topic namespace with the specified target namespace.
    *
    * @param targetNs DOCUMENT_ME
    */
   public TopicNamespaceImpl( String targetNs )
   {
      if ( targetNs == null )
      {
         throw new IllegalArgumentException( "Target namespace must not be null (\"\" should be used to represent the default namespace)." );
      }

      m_targetNs = targetNs;
   }

   /**
    * DOCUMENT_ME
    *
    * @param finality DOCUMENT_ME
    */
   public void setFinal( boolean finality )
   {
      m_final = finality;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean isFinal(  )
   {
      return m_final;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void setName( String name )
   {
      m_name = name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getName(  )
   {
      return m_name;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getTargetNamespace(  )
   {
      return m_targetNs;
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Topic getTopic( String name )
   {
      return (Topic) m_rootTopicMap.get( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Topic addTopic( String name )
   {
      return addTopic( new TopicImpl( name ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean containsTopic( String name )
   {
      return m_rootTopicMap.containsKey( name );
   }

   /**
    * DOCUMENT_ME
    *
    * @param obj DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean equals( Object obj )
   {
      return ( obj != null ) && obj instanceof TopicNamespaceImpl && ( obj.hashCode(  ) == hashCode(  ) );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public int hashCode(  )
   {
      return m_targetNs.hashCode(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param name DOCUMENT_ME
    */
   public void removeTopic( String name )
   {
      m_rootTopicMap.remove( name );
   }

   /**
    * Returns a String representation of this TopicNamespace in following format:
    * "{" + target_namespace + "}" + name. If the target namespace is
    * "" (i.e. the ad-hoc namespace), only the name is returned. If the name is
    * null, the name is ommitted.
    * For example: "{http://example.org/topicSpace/example1}TopicSpaceExample1" or
    * "AdHocTopicSpace".
    *
    * @return a String representation of this TopicNamespace
    */
   public String toString(  )
   {
      StringBuffer strBuf = new StringBuffer(  );
      if ( m_targetNs != null )
      {
         strBuf.append( '{' );
         strBuf.append( m_targetNs );
         strBuf.append( '}' );
      }

      if ( m_name != null )
      {
         strBuf.append( m_name );
      }

      return strBuf.toString(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Iterator topicIterator(  )
   {
      return m_rootTopicMap.values(  ).iterator(  );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topic DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   private Topic addTopic( Topic topic )
   {
      m_rootTopicMap.put( topic.getName(  ),
                          topic );
      topic.setTopicNamespace( this );
      List topicPath = new LinkedList(  );
      topicPath.add( topic.getName(  ) );
      topic.setTopicPath( topicPath );
      return topic;
   }
}