/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.impl;

import org.apache.ws.notification.WsnRuntime;
import org.apache.ws.notification.topics.Topic;
import org.apache.ws.notification.topics.TopicSet;
import org.apache.ws.notification.topics.expression.TopicExpression;
import org.apache.ws.notification.topics.expression.TopicExpressionException;
import javax.xml.namespace.QName;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

/**
 * An implementation of a <code>TopicSet</code>.
 *
 * @author Ian Springer
 */
public class TopicSetImpl
   implements TopicSet
{
   private Set     m_topicExprs = Collections.synchronizedSet( new HashSet(  ) );
   private Map     m_topics = Collections.synchronizedMap( new HashMap(  ) ); // maps topicPath QName to Topic
   private boolean m_fixed;

   /**
    * Creates a new {@link TopicSetImpl} object.
    */
   public TopicSetImpl(  )
   {
   }

   /**
    * Creates a new {@link TopicSetImpl} object.
    *
    * @param topicExprs DOCUMENT_ME
    *
    * @throws TopicExpressionException DOCUMENT_ME
    */
   public TopicSetImpl( Set topicExprs )
   throws TopicExpressionException
   {
      for ( Iterator iterator = topicExprs.iterator(  ); iterator.hasNext(  ); )
      {
         TopicExpression topicExpr = (TopicExpression) iterator.next(  );
         addTopicExpression( topicExpr );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param fixed DOCUMENT_ME
    */
   public void setFixed( boolean fixed )
   {
      m_fixed = fixed;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean isFixed(  )
   {
      return m_fixed;
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public TopicExpression[] getTopicExpressions(  )
   {
      return (TopicExpression[]) m_topicExprs.toArray( new TopicExpression[m_topicExprs.size(  )] );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public Topic[] getTopics(  )
   {
      return (Topic[]) m_topics.values(  ).toArray( new Topic[m_topics.size(  )] );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicExpr DOCUMENT_ME
    *
    * @throws TopicExpressionException DOCUMENT_ME
    */
   public void addTopicExpression( TopicExpression topicExpr )
   throws TopicExpressionException
   {
      Topic[] topics = WsnRuntime.getRuntime(  ).getTopicExpressionEngine(  ).evaluateTopicExpression( topicExpr );
      m_topicExprs.add( topicExpr );
      for ( int i = 0; i < topics.length; i++ )
      {
         m_topics.put( QName.valueOf( topics[i].toString(  ) ),
                       topics[i] );
      }
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicPath DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public boolean containsTopic( QName topicPath )
   {
      return m_topics.containsKey( topicPath );
   }

   /**
    * DOCUMENT_ME
    *
    * @param topicExpr DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    *
    * @throws TopicExpressionException DOCUMENT_ME
    */
   public Topic[] evaluateTopicExpression( TopicExpression topicExpr )
   throws TopicExpressionException
   {
      Topic[] topics = WsnRuntime.getRuntime(  ).getTopicExpressionEngine(  ).evaluateTopicExpression( topicExpr );
      return getSupportedTopics( topics );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String toString(  )
   {
      return m_topics.toString(  );
   }

   private Topic[] getSupportedTopics( Topic[] topics )
   {
      List supportedTopics = new ArrayList(  );
      for ( int i = 0; i < topics.length; i++ )
      {
         Topic topic = topics[i];
         if ( containsTopic( QName.valueOf( topic.toString(  ) ) ) )
         {
            supportedTopics.add( topic );
         }
      }

      return (Topic[]) supportedTopics.toArray( new Topic[supportedTopics.size(  )] );
   }
}