/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.notification.topics.v2004_06;

import org.apache.ws.pubsub.Filter;
import org.apache.ws.util.XmlBeanUtils;
import org.oasisOpen.docs.wsn.x2004.x06.wsnWSBaseNotification12Draft01.TopicExpressionType;
import java.net.URI;
import java.net.URISyntaxException;

/**
 * Provides an Abstract base class for TopicExpressions
 *
 * @author Ian Springer (ian DOT springer AT hp DOT com)
 */
public abstract class AbstractTopicExpression
   implements Filter
{
   private String m_expr;
   private URI    m_dialect;

   /**
    * Creates a new {@link AbstractTopicExpression} object.
    *
    * @param topicExprXBean DOCUMENT_ME
    */
   public AbstractTopicExpression( TopicExpressionType topicExprXBean )
   {
      m_expr = XmlBeanUtils.getValue( topicExprXBean );
      try
      {
         m_dialect = new URI( topicExprXBean.getDialect(  ) );
      }
      catch ( URISyntaxException urise )
      {
         throw new RuntimeException( urise );
      }
   }

   /**
    * Returna the URI for the dialect associated with this topic expression
    *
    * @return  dialect {@link URI}
    */
   public URI getDialect(  )
   {
      return m_dialect;
   }

   /**
    * Returns the expression object associated with this topic expression.
    *
    * @return expression object.
    */
   public Object getExpression(  )
   {
      return m_expr;
   }
}