/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.pubsub;

import org.apache.ws.notification.topics.Topic;
import org.w3c.dom.Node;
import java.net.URI;

/**
 *
 * @author  Stefan Lischke
 */
public class TopicFilter
   extends AbstractFilter
   implements Filter
{
   private String m_targetNamespace;

   /**
    * Creates a new {@link TopicFilter} object.
    *
    * @param t DOCUMENT_ME
    */
   public TopicFilter( Topic t )
   {
      String             topicexpr = "";
      java.util.Iterator it = t.getTopicPath(  ).iterator(  );
      while ( it.hasNext(  ) )
      {
         String tt = (String) it.next(  );
         topicexpr += ( "tns:" + tt );
         if ( it.hasNext(  ) )
         {
            topicexpr += "/";
         }
      }

      m_targetNamespace = t.getTopicNamespace(  ).getTargetNamespace(  ).toString(  );
      super.setURI( FilterFactory.topicsuri );
      super.setContent( parse( topicexpr ) );
   }

   /**
    * Creates a new {@link TopicFilter} object.
    *
    * @param filter DOCUMENT_ME
    */
   public TopicFilter( String filter )
   {
      super.setContent( parse( filter ) );
      super.setURI( FilterFactory.topicsuri );
   }

   /**
    * Creates a new {@link TopicFilter} object.
    *
    * @param content DOCUMENT_ME
    * @param dialect DOCUMENT_ME
    */
   public TopicFilter( Node content,
                       URI  dialect )
   {
      super( content, dialect );
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public String getNameSpace(  )
   {
      return m_targetNamespace;
   }

   private Node parse( String s )
   {
      javax.xml.parsers.DocumentBuilderFactory dbf = javax.xml.parsers.DocumentBuilderFactory.newInstance(  );
      try
      {
         javax.xml.parsers.DocumentBuilder db  = dbf.newDocumentBuilder(  );
         org.w3c.dom.Document              dom = db.newDocument(  );
         return ( dom.createTextNode( s ) );
      }
      catch ( javax.xml.parsers.ParserConfigurationException e )
      {
         //TODO
         e.printStackTrace(  );
      }

      return null;
   }
}