/*=============================================================================*
 *  Copyright 2004 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *=============================================================================*/
package org.apache.ws.pubsub.emitter;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 * The engine for configuring SOAP message emitters for various protocols and
 * spawning emitter tasks.
 *
 * @author Ian Springer
 */
public class SoapMessageEmitterEngine
{
   private static final SoapMessageEmitterEngine INSTANCE = new SoapMessageEmitterEngine(  );

   /*
    * <String>protocol -> <SoapMessageEmitter>emitter
    */
   private final Map m_emitterMap = new HashMap(  );

   private SoapMessageEmitterEngine(  )
   {
   }

   /**
    * DOCUMENT_ME
    *
    * @return DOCUMENT_ME
    */
   public static SoapMessageEmitterEngine getInstance(  )
   {
      return INSTANCE;
   }

   /**
    * Loads each of the TopicNamespace documents from the URLs in the specified set and
    * adds them to this registry.
    *
    * @param emitterMap a set of Strings representing class names of {@link EmitterTask}s
    */
   public synchronized void setEmitterMap( Map emitterMap )
   {
      for ( Iterator iterator = emitterMap.keySet(  ).iterator(  ); iterator.hasNext(  ); )
      {
         String protocol         = (String) iterator.next(  );
         String emitterClassName = (String) emitterMap.get( protocol );
         Class  emitterClass;
         try
         {
            emitterClass = Class.forName( emitterClassName );
         }
         catch ( ClassNotFoundException cnfe )
         {
            throw new RuntimeException( cnfe );
         }

         if ( !( emitterClass.isAssignableFrom( SoapMessageEmitter.class ) ) )
         {
            throw new RuntimeException( "Class " + emitterClass.getName(  ) + " does not implement the "
                                        + SoapMessageEmitter.class.getName(  ) + " interface." );
         }

         m_emitterMap.put( protocol, emitterClass );
      }
   }
}